/*****************************************************
Project : UselessBox
Version : 1.1
Date    : 08.11.2020
Author  : RA4NAL
Company : Home

Chip type               : ATmega48
AVR Core Clock frequency: 8,000000 MHz
Memory model            : Small
External RAM size       : 0
Data Stack size         : 128

Ports connections:
PB  - All IN PullUP  Not used

PC0 - LED R OUT 1 (Activ Low)
PC1 - LED G OUT 1 (Activ Low)
PC2 - LED Y OUT 1 (Activ Low)
PC3...PC5  - All IN PullUP  Not used

PD0 - IN PullUP  USART Not used
PD1 - IN PullUP  USART Not used
PD2 - IN PullUP  INT0 (switch "0")
PD3 - IN PullUP  INT1 (switch "ON")
PD4 - OUT 1 ( =0 - Revers ON)
PD5 - OUT 0 ( =1 Forward ON)
PD6 - OUT 0 ( =1 Revers ON)
PD7 - OUT 1 ( =0 - Forward ON)

*****************************************************/
// CASE 1,2: Time for open cover
#define TimeOpenMul 3   // TimeGoFul * (3/5) 
#define TimeOpenDiv 5   // //////////////// 
// CASE 3:  Time for move, but not open cover
#define TimeNotOpenDiv  4   // TimeGoFul * (1/4) 

// Time for quick press switch
#define QuickTime 10      // 0...FF for TCNT1H

// Number of cases in switch
#define MaxCase 7         // case 0 ... case 7

#define Disconnect  0x9F  // 1001.1111 PORTD - disconnect motor
#define Revers 0xCF       // 1100.1111 PORTD - Revers Max Speed
#define Forward 0x3F      // 0011.1111 PORTD - Forward Max Speed

#include <mega48.h>
#include <delay.h>

// Global variables
unsigned char State;      // Number of effect
unsigned char StateNew;   // Random from 0 to MaxCase
unsigned char StateBut, cnt0, cnt1; // for interrupt TMR0
unsigned int  TimeGoFul;

interrupt [EXT_INT0] void inter0(void)
 { 
   EIMSK = 0x02;      // Exit from Sleep, Desable INT0, inable INT1
 }

interrupt [EXT_INT1] void inter1(void)
 { 
   EIMSK = 0x01;      // Exit from Sleep, Desable INT1, inable INT0
 }

interrupt [TIM0_OVF] void intTMR0(void)
 {
  unsigned char delta, sample, toggle;

  sample = PIND & 0x0C;               // 0000.1100 PIND2, PIND3 only
// Debouncing buttons with vertical counter
  delta = sample ^ StateBut;
  cnt1 = (cnt1 ^ cnt0) & delta;
  cnt0 = ~cnt0 & delta;
  toggle = delta & ~(cnt0 | cnt1);
  StateBut ^= toggle;                // debouncing buttons
  StateNew +=1;
  if (StateNew > MaxCase) StateNew = 0;  // Random from 0 to MaxCase
 }

void startTMR1(unsigned int i)
 {
   unsigned char tmp;
   
   TCNT1H=0x00;       // TMR1=0000
   TCNT1L=0x00;       // /////////
   TCCR1B=0x05;       // start TMR1
   tmp = i>>8;        // High byte
   OCR1AH = tmp;
   OCR1AL = (char) i; // Low byte
   TIFR1.OCF1A=1;     // Clear flag OCF1A
   TIFR1.TOV1=1;      // Clear flag TOV1
 }

void startTMR0(void)
 {
   TCNT0=0x00;
   TIFR0=0x01;         // Clear TIFR0.TOV0
   TIMSK0=0x01;        // Inable TMR0 Overflow interrupt 
   TCCR0B=0x03;        // Start TMR0 DIV=64, Tovf = 8000/256/64 = ~0,5 KHz (2ms)
 }
 
void gosleep(void)
 {
   #asm
   sei           ; Global enable interrupts
   #endasm
   PORTC=0x7F;        // All LEDs off
   PORTD=Disconnect;  // Disconnect motor
   DDRD=0xF0;         // /////////
   startTMR0();  
   delay_ms (50);    // for read buttons
   startTMR1(0xFFFF);
   if ((StateBut & 0x04) == 0)   // PIND.2 = 0, Not 0 state
     {
      PORTD = Revers;  // Start revers
      while ((TIFR1.TOV1 == 0) & ((StateBut & 0x04) == 0)); // wait PIND2=1 max 8 sec
      PORTD = Disconnect;  // Disconnect motor
      delay_ms (100);
     }
   while ((StateBut & 0x08) == 0); // wait for switch off (PIND3=1)
    {           
      delay_ms (100);
    }
   TCCR1B = 0x00;     // stop TMR1
   TIFR1.TOV1=1;      // Reset flag overflow TMR1
   TCCR0A=0x00;       // stop TMR0
   TCCR0B=0x00;       // ////////// 
   TIFR0.TOV0=1;      // Reset flag TOV0
   EICRA = 0x00;      // INT0, INT1 - Low level
   EIMSK = 0x03;      // Inable INT0, INT1
   SMCR = 0x05;       // 0000.0101 Inable PowerDwn
   #asm
     sleep
   #endasm
   
// Exit from sleep   
   SMCR = 0x04;       // 0000.0100 Disable Sleep
   EIMSK = 0x00;      // Desable INT0, INT1
   State = 0x00;
   StateBut = 0x0C;   // Buttons PIND2, PIND3
   cnt0 = 0;          // For Read Buttons
   cnt1 = 0;          // ///////////////
   startTMR0();
   #asm
   sei           ; Global enable interrupts
   #endasm
   delay_ms (100);
   startTMR1(0xFFFF);
 }

void main(void)
{
// Declare your local variables here

#include "initmega48.c"        // Init SFR mega48
  gosleep();

while (1)
  {

   if (TIFR1.TOV1)        // TimeOut
     {
       gosleep();
     }
   if ((StateBut & 0x08) == 0)   // PIND.3 = 0, Switch
    { 
       unsigned char tmp;
       
       tmp = TCNT1L;
       tmp = TCNT1H;
       if (tmp < QuickTime) // Push time < QuickTime, State=0
         {
           State = 0x00;
         }
       else                 // Push time >= QuickTime, state 1...
         {
           State = StateNew;// Random from 0 to MaxCase
         }
     

      switch (State)
      {
       case 7: delay_ms (800);
       
       case 0:  // open end switch
         { 
           unsigned char tmp;
           unsigned int tmp1;

           delay_ms (500);
           if ((StateBut & 0x04) == 0)   // PIND.2 = 0, Not 0 state
            {
              PORTD = Revers;       // Start revers
              startTMR1(0xFFFF);
              while ((TIFR1.TOV1 == 0) & ((StateBut & 0x04) == 0)); // wait PIND2=1 max 8 sec
              PORTD = Disconnect;  // Disconnect motor
            }
           PORTC.1 = 0;         // Green LED ON
           delay_ms (30);
           PORTD = Forward;     // Start forward
           startTMR1(0xFFFF);
           while ((TIFR1.TOV1 == 0) & ((StateBut & 0x08) == 0)); // wait PIND3=1 max 8 sec
           PORTD = Disconnect;  // Disconnect motor
           tmp = TCNT1L;
           tmp1 = TCNT1H;
           TimeGoFul = (tmp1 <<8) + tmp;
           delay_ms (80);
           PORTC=0x7F;          // All LEDs off
           PORTD = Revers;      // Start revers
           while ((TIFR1.TOV1 == 0) & ((StateBut & 0x04) == 0)); // wait PIND2=1 max 8 sec
           PORTD = Disconnect; // Disconnect motor
           delay_ms (200);
           TCNT1H=0x00;        // TMR1=0000
           TCNT1L=0x00;        // /////////
           break;
         }

       case 6: delay_ms (800);

       case 1:   // Open, wait, close, open and switch
         {
           unsigned int tmp;
           
           tmp = (TimeGoFul / TimeOpenDiv) * TimeOpenMul;
           delay_ms (500);
           PORTC.1 = 0;          // Green LED ON
           startTMR1(tmp);
           PORTD = Forward;      // Start forward   
           while (TIFR1.OCF1A == 0); // wait for open cover
           PORTD = Disconnect;   // Disconnect motor
           delay_ms (1000);
           PORTD = Revers;       // Start revers
           while ((TIFR1.TOV1 == 0) & ((StateBut & 0x04) == 0)); // wait PIND2=1 max 8 sec
           PORTD = Disconnect;   // Disconnect motor
           delay_ms (50);
           PORTD = Forward;     // Start forward
           startTMR1(0xFFFF);
           while ((TIFR1.TOV1 == 0) & ((StateBut & 0x08) == 0)); // wait PIND3=1 max 8 sec
           PORTD = Disconnect;  // Disconnect motor
           delay_ms (100);
           PORTD = Revers;  // Start revers
           while ((TIFR1.TOV1 == 0) & ((StateBut & 0x04) == 0)); // wait PIND2=1 max 8 sec
           PORTD = Disconnect;  // Disconnect motor
           PORTC=0x7F;          // All LEDs off
           delay_ms (200);
           TCNT1H=0x00;         // TMR1=0000
           TCNT1L=0x00;         // /////////
           break;
         }

       case 2:   // open, shake, switch
         {
           unsigned int tmp;
           unsigned char i;
           
           tmp = (TimeGoFul / TimeOpenDiv) * TimeOpenMul;
           delay_ms (800);
           startTMR1(tmp);
           PORTC.0 = 0;              // Red LED ON
           PORTD = Forward;          // Start forward         
           while (TIFR1.OCF1A == 0); // wait for open cover
           PORTD = Disconnect;       // Disconnect motor
           delay_ms (200);
           for (i=0; i<2; i+=1)
             {
               PORTC.0 = 1;          // Red LED OFF
               PORTC.2 = 0;          // Yellow LED ON
               startTMR1(0xFFFF);
               PORTD = Revers;       // Start revers
               while ((TIFR1.TOV1 == 0) & ((StateBut & 0x04) == 0)); // wait PIND2=1 max 8 sec
               PORTD = Disconnect;   // Disconnect motor
               delay_ms (100);
               PORTC.0 = 0;          // Red LED ON
               PORTC.2 = 1;          // Yellow LED OFF
               startTMR1(tmp);
               PORTD = Forward;      // Start forward    
               while (TIFR1.OCF1A == 0); // wait for open cover
               PORTD = Disconnect;   // Disconnect motor
               delay_ms (100);
             }
           startTMR1(0xFFFF);
           PORTD = Revers;       // Start revers
           while ((TIFR1.TOV1 == 0) & ((StateBut & 0x04) == 0)); // wait PIND2=1 max 8 sec
           PORTD = Disconnect;  // Disconnect motor
           delay_ms (1000);
           PORTD = Forward;     // Start forward
           startTMR1(0xFFFF);
           while ((TIFR1.TOV1 == 0) & ((StateBut & 0x08) == 0)); // wait PIND3=1 max 8 sec
           PORTD = Disconnect;  // Disconnect motor
           delay_ms (100);
           PORTD = Revers;      // Start revers
           while ((TIFR1.TOV1 == 0) & ((StateBut & 0x04) == 0)); // wait PIND2=1 max 8 sec
           PORTD = Disconnect;  // Disconnect motor
           PORTC=0x7F;          // All LEDs off
           delay_ms (200);
           TCNT1H=0x00;         // TMR1=0000
           TCNT1L=0x00;         // /////////
           break;
         }

       case 3:   // shake, wait and switch
         {
           unsigned int tmp;
           unsigned char i;
           
           tmp = TimeGoFul / TimeNotOpenDiv;
           delay_ms (800);
           startTMR1(tmp);
           PORTC.0 = 0;              // Red LED ON
           PORTD = Forward;          // Start forward
           while (TIFR1.OCF1A == 0); // wait for open cover
           PORTD = Disconnect;       // Disconnect motor
           delay_ms (800);
           for (i=0; i<3; i+=1)
             {
               startTMR1(0xFFFF);
               PORTD = Revers;       // Start revers
               while ((TIFR1.TOV1 == 0) & ((StateBut & 0x04) == 0)); // wait PIND2=1 max 8 sec
               PORTD = Disconnect;   // Disconnect motor
               delay_ms (100);
               startTMR1(tmp);
               PORTD = Forward;      // Start forward   
               while (TIFR1.OCF1A == 0); // wait for open cover
               PORTD = Disconnect;   // Disconnect motor
               delay_ms (100);
             }
           startTMR1(0xFFFF);
           PORTD = Revers;        // Start revers
           while ((TIFR1.TOV1 == 0) & ((StateBut & 0x04) == 0)); // wait PIND2=1 max 8 sec
           PORTD = Disconnect;   // Disconnect motor
           delay_ms (100);
           PORTD = Forward;      // Start forward
           startTMR1(0xFFFF);
           PORTC.0 = 1;          // Red LED OFF
           PORTC.2 = 0;          // Yellow LED ON
           while ((TIFR1.TOV1 == 0) & ((StateBut & 0x08) == 0)); // wait PIND3=1 max 8 sec
           PORTD = Disconnect;  // Disconnect motor
           delay_ms (100);
           PORTD = Revers;      // Start revers
           while ((TIFR1.TOV1 == 0) & ((StateBut & 0x04) == 0)); // wait PIND2=1 max 8 sec
           PORTD = Disconnect;  // Disconnect motor
           PORTC=0x7F;          // All LEDs off
           delay_ms (200);
           TCNT1H=0x00;         // TMR1=0000
           TCNT1L=0x00;         // /////////
           break;
         }

       case 5: delay_ms (800);

       case 4:  // open, wait end switch
         { 
           delay_ms (500);
           PORTC.0 = 0;         // Red LED ON
           PORTD = Forward;     // Start forward
           startTMR1(0xFFFF);
           while ((TIFR1.TOV1 == 0) & ((StateBut & 0x08) == 0)); // wait PIND3=1 max 8 sec
           PORTD = Disconnect;  // Disconnect motor
           delay_ms (800);
           PORTC.0 = 1;          // Red LED OFF
           PORTC.2 = 0;          // Yellow LED ON
           delay_ms (200);
           PORTD = Revers;      // Start revers
           while ((TIFR1.TOV1 == 0) & ((StateBut & 0x04) == 0)); // wait PIND2=1 max 8 sec
           PORTD = Disconnect; // Disconnect motor
           PORTC=0x7F;          // All LEDs off
           delay_ms (200);
           TCNT1H=0x00;        // TMR1=0000
           TCNT1L=0x00;        // /////////
           break;
         }
      }
    }
  }
}
